unit processThreadUnit;
{
    This unit contains main task work that should be run
    under a specific thread (to display correctly the work in progress
    and to give control back to the user during this long process)
______________________________________________________________________________
}
{TurboVOB - A freeware to prepare DTH7000's video for DVD burning on computer under windows.
Copyright (C) 2005 Didier CADIOU

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see gpl.txt file); if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.}

interface

uses
  //personal :
  task, globalMethods,
  //generic :
  Classes, SyncObjs, SysUtils, Dialogs, language, Windows, DateUtils,
  strUtils, forms;

type TErrorEvent = procedure (Sender: TObject; ErrorString: string; taskNber: integer) of object;
type TProgressEvent = procedure (Sender: TObject; progressString: string; taskNber: integer) of object;
{type TProgressEvent2 = procedure (Sender: TObject;
    progressString: string;
    taskNber: integer;
    destination: string;
    expectedSize: real) of object; }
type

  TprocessThread = class(TThread)
  private
    FEvent : TEvent;  { internal event }
    FOnCmdStarted: TProgressEvent;  {event Procedure that will be called each time a cmd line is started}
    FOnCmdDone: TProgressEvent;  {event Procedure that will be called each time a cmd line is done}
    FOnTaskStarted: TProgressEvent;  {event Procedure that will be called each time a task is started}
    FOnError: TErrorEvent;  {event Procedure to be called when an error occurs}
    tasksSet: TAllTask; //collection of all task to run here
    refSize: real; //reference file size from which expected size will be estimated (used to check file size after each command)
    function RunCmdLine(const CmdLine: string; WindowState: Word = SW_NORMAL): Boolean; //launch an external program with the desired command line
    function Run1Command(cmdLine: string): Boolean; //start 1 command line and manage messages (use RunCmdLine)
    function isEnoughSpaceFor(sourcePath: string; destPath: string): boolean; //check that free space is sufficiant to run task
    function isFileSizeCorrect(theFileName: string; expectedSizeKo: real; tolerance: real): boolean; //check no error occured by checking the filesize after operation
    function isDirectorySizeCorrect(theDirName: string; expectedSizeKo: real; tolerance: real): boolean; //idem but for video_ts directory
    function Run1DemuxingTask(theTask: ttask): boolean;
    function Run1RemuxingTask(theTask: ttask): boolean;
    function Run1AuthoringTask(theTask: ttask): boolean;

  protected
    procedure TellCmdStarted(ProgressMessage: string; taskNber: integer); { Wrapper for FOnCmdStarted}
    procedure TellCmdDone(ProgressMessage: string; taskNber: integer); { Wrapper for FOnCmdDone}
    procedure TellTaskStarted(ProgressMessage: string; taskNber: integer); { Wrapper for FOnTaskStarted}
    procedure TellErrorOccurs(errorMessage: string; taskNber: integer); { Wrapper for FOnError}
    procedure Run1Task(theTask: ttask); //define the command line and start it (use Run1Command, then RunCmdLine)
    procedure Execute; override;
  public
    taskNber: integer; //Nber of the task currently in progress
    progressComment: string; //comment which will be displayed while task operation is in progress
    expectedSize: real; //of the destination in Ko (used to control error is missing and to update progress bar)
    destination: string; //path of the destination file (or dir VIDEO_TS)
    isDestinationAdir: boolean; //true if the destination is a directory
    constructor Create(CreateSuspended: Boolean; TaskList: TAllTask);
    //events fired regularely during process :
    property OnCmdStarted: TProgressEvent read FOnCmdStarted write FOnCmdStarted; {when a cmd line has been started}
    property OnCmdDone: TProgressEvent read FOnCmdDone write FOnCmdDone; {when a cmd line has been ended}
    property OnTaskStarted: TProgressEvent read FOnTaskStarted write FOnTaskStarted; {before all cmd line for this task will be started}
    property OnError : TErrorEvent read FOnError write FOnError; { property to enable event OnError management}
    destructor Destroy();
  end;

implementation
uses main;

//--------------------------------------------------------------------------------------------------------
constructor TProcessThread.Create(CreateSuspended: Boolean; TaskList: TAllTask); //list of unwanted segment inside the video

{

parameters :
    thDrv = drive number
    thID = ID section number related to the saved video
    thFullPathName = destination folder + filename
}
begin
    inherited Create(CreateSuspended);
    Priority := tpLowest;
    FEvent := TEvent.Create(nil, False, False, '');
    tasksSet := TaskList;

end;

//--------------------------------------------------------------------------------------------------------
procedure TProcessThread.Execute;
var
    ix: integer; //loop index
begin

    for ix := 0 to tasksSet.Count-1 do
    begin
        TellTaskStarted('',ix); //fire the appropriate event to display work in progress
        taskNber := ix;
        run1Task(tasksSet.Tasks[ix]); //process the task
        if terminated = true then exit;
    end;

end;


//--------------------------------------------------------------------------------------------------------
procedure TProcessThread.Run1Task(theTask: ttask);
//(automatically executed once this thread is created)
var
    taskOK: boolean; //true if task succeeded
begin

    if theTask.ToBeDemuxed = true then begin
        taskOK := Run1DemuxingTask(theTask);
        if taskOK = false then exit; //do not continue if previous work fails
    end; //to be demuxed ?


    if theTask.ToBeRemuxed = true then begin
        taskOK := Run1RemuxingTask(theTask);
        if taskOK = false then exit; //do not continue if previous work fails
    end; //to be remuxed ?


    if theTask.ToBeAuthored = true then begin
        taskOK := Run1AuthoringTask(theTask);
        if taskOK = false then exit; //do not continue if previous work fails
    end; //To Be Authored ?
end;


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function TProcessThread.Run1DemuxingTask(theTask: ttask): boolean;
 var
    cmdLineOK: boolean; //true if cmd line succeeded
    cmd1: string; //current command line to execute
 begin
        result := false; //until demonstration has been done that everything wroked correctly

        //check that file exists :
        if FileExists(theTask.originalVobPath) = false then begin
            TellErrorOccurs(theTask.originalVobPath + rstrFILE_DOESNOT_EXITS, taskNber);
            exit;
        end;

        refSize := getSizeOfFile(theTask.originalVobPath); //use the source file size as reference

        //check free space :
        if isEnoughSpaceFor(theTask.originalVobPath, theTask.videoStreamPath) = false then exit;


        //demux the audio stream : -----------------
        expectedSize := 0.065 * refSize; //audio stream is between 3% and 9% of full VOB (depending on compression ratio which only affects videos)
        destination := theTask.audioStreamPath;
        isDestinationAdir := false; //destination is a file

        cmd1 := '"' + optDmuxExePath + '" "' +      //demultiplexer location
            theTask.originalVobPath + '" 0xbd "' +  //in: original vob file location + substream ID
            theTask.audioStreamPath + '"' +         //out: destination audio file location
            ' 0x80';                                //audio substream ID
        TellCmdStarted(rstrAUDIO_DEMUX_IN_PROGRESS + #13#10'    ' + cmd1, taskNber);
        cmdLineOK := Run1Command(cmd1);
        TellCmdDone(rstrTASK_DONE, taskNber); //fire one progression message
        if cmdLineOK = false then exit;
        isFileSizeCorrect(destination, expectedSize, 60);
        if terminated = true then exit;


        //demux the video stream : -----------------
        expectedSize := 0.90 * refSize; //video stream size is close to the full VOB file size (depending on compression ratio)
        destination := theTask.videoStreamPath;
        isDestinationAdir := false; //destination is a file

        cmd1 := '"' + optDmuxExePath + '" "' +      //demultiplexer location
            theTask.originalVobPath + '" 0xe0 "' +  //in: original vob file location + video stream ID
            theTask.videoStreamPath + '"';          //out: destination video file location
        TellCmdStarted(rstrVIDEO_DEMUX_IN_PROGRESS + #13#10'    ' + cmd1, taskNber);
        cmdLineOK := Run1Command(cmd1);
        TellCmdDone(rstrTASK_DONE, taskNber); //fire one progression message
        if cmdLineOK = false then exit;
        isFileSizeCorrect(destination, expectedSize, 5);
        if terminated = true then exit;

        if theTask.deleteOriginalVob = true then begin
            DeleteFile(Pchar(theTask.originalVobPath));
            TellCmdDone(theTask.originalVobPath + rstrDELETED, taskNber);
        end;

        result := true; //if everything before succeeded
 end;



//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function TProcessThread.Run1RemuxingTask(theTask: ttask): boolean;
 var
    cmdLineOK: boolean; //true if cmd line succeeded
    cmd1: string; //current command line to execute
 begin
        result := false; //until demonstration has been done that everything wroked correctly

        //check that files exists :
        if FileExists(theTask.videoStreamPath) = false then begin
            TellErrorOccurs(theTask.videoStreamPath + rstrFILE_DOESNOT_EXITS, taskNber);
            exit;
        end;
        if FileExists(theTask.audioStreamPath) = false then begin
            TellErrorOccurs(theTask.audioStreamPath + rstrFILE_DOESNOT_EXITS, taskNber);
            exit;
        end;

        //calculate ref size if streams are sources :
        if theTask.ToBeDemuxed = false then begin //if streams are sources
            refSize := 1.04 * (getSizeOfFile(theTask.videoStreamPath) +
                getSizeOfFile(theTask.audioStreamPath)); //calculate from the summation
                    //of both streams, increased by an approx. factor
        end;

        //check free space :
        if isEnoughSpaceFor(theTask.videoStreamPath, theTask.VobPath) = false then exit;

        //remux streams together to create the vob file : -------------------
        expectedSize := refSize; //remuxed VOB file size should be similar to size before demux
        destination := theTask.VobPath;
        isDestinationAdir := false; //destination is a file

        cmd1 := '"' + optMuxExePath + '" -f 8 -V -o' +  //remultiplexer location + options
            ' "' + theTask.VobPath + '"' +              //out: destination file (vob)
            ' "' + theTask.videoStreamPath + '"' +      //in: video stream (.m2v)
            ' "' + theTask.audioStreamPath + '"';       //in: audio stream (.ac3)
        TellCmdStarted(rstrREMUX_IN_PROGRESS + #13#10'    ' + cmd1, taskNber);
        cmdLineOK := Run1Command(cmd1);
        TellCmdDone(rstrTASK_DONE, taskNber); //fire one progression message
        if cmdLineOK = false then exit;
        isFileSizeCorrect(destination, expectedSize, 5);
        if terminated = true then exit;

        if theTask.deleteStreams = true then begin
            DeleteFile(Pchar(theTask.videoStreamPath));
            TellCmdDone(theTask.videoStreamPath + rstrDELETED, taskNber);
            DeleteFile(Pchar(theTask.audioStreamPath));
            TellCmdDone(theTask.audioStreamPath + rstrDELETED, taskNber);
        end;

        result := true; //if everything before succeeded
 end;


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function TProcessThread.Run1AuthoringTask(theTask: ttask): boolean;
 var
    cmdLineOK: boolean; //true if cmd line succeeded
    cmd1: string; //current command line to execute
 begin
        result := false; //until demonstration has been done that everything wroked correctly

        //check that files exists :
        if FileExists(theTask.VobPath) = false then begin
            TellErrorOccurs(theTask.VobPath + rstrFILE_DOESNOT_EXITS, taskNber);
            exit;
        end;

        //calculate ref size if remuxed VOB is the source :
        if (theTask.ToBeDemuxed = false) and
          (theTask.ToBeRemuxed = false) then begin //if remuxed VOB file is the source
            refSize := getSizeOfFile(theTask.VobPath); //use the source file size as reference
        end;


        //check free space :
        if isEnoughSpaceFor(theTask.VobPath, theTask.destinationFolder) = false then exit;

        //cmdline example :
        //"C:\Program Files\DVDAuthorGUI\bin\dvdauthor.exe" -o "C:\VOBs\D Avent. Hum. & Geo" -t -c 0,00:05:00,00:10:00 "C:\VOBs\D Avent. Hum. & Geo\remuxedVTS_01_1.VOB"


        //author the DVD title set (VTS_01_0.IFO & BUP files file creation) : ------------------
        expectedSize := refSize; //VIDEO_TS folder size should be similar to size of original VOB file (IFO files are very small and can be neglected)
        destination := theTask.destinationFolder + '\VIDEO_TS\';
        isDestinationAdir := true; //destination is a folder

        if length(theTask.chapterListStr)>0 then begin //cmd line definition with chapters list
            cmd1 := '"' + optAuthorExePath + '" -o' +       //re-author program location
                ' "' + theTask.destinationFolder  + '"' +   //out: destination DVD folder
                ' -t -c ' + theTask.chapterListStr +        //in: chapters list (expressed in time)
                ' -f "' +theTask.VobPath + '"';                //in: the vob file
        end else begin //cmd line definition without chapters list
            cmd1 := '"' + optAuthorExePath + '" -o' +       //re-author program location
                ' "' + theTask.destinationFolder  + '"' +   //out: destination DVD folder
                ' -t -f "' + theTask.VobPath + '"';            //in: the vob file
                    //(-f added due to error message = bad version of getopt: please precede all sources with -f)
        end;
        TellCmdStarted(rstrTITLE_AUTHORING_IN_PROGRESS + #13#10'    ' + cmd1, taskNber);
        cmdLineOK := Run1Command(cmd1);
        TellCmdDone(rstrTASK_DONE, taskNber); //fire one progression message
        if cmdLineOK = false then exit;
        isDirectorySizeCorrect(destination, expectedSize, 5);
        if terminated = true then exit;


        //author the global DVD (VIDEO_TS.IFO & BUP files creation) : -----------------
        cmd1 := '"' + optAuthorExePath + '" -o' +       //re-author program location
            ' "' + theTask.destinationFolder  + '" -T'; //out: destination DVD folder
        TellCmdStarted(rstrFOLDER_AUTHORING_IN_PROGRESS + #13#10'    ' + cmd1, taskNber);
        cmdLineOK := Run1Command(cmd1);
        TellCmdDone(rstrTASK_DONE, taskNber); //fire one progression message
        if cmdLineOK = false then exit;
        if terminated = true then exit;

        if theTask.deleteRemuxedVob = true then begin
            DeleteFile(Pchar(theTask.VobPath));
            TellCmdDone(theTask.VobPath + rstrDELETED, taskNber);
        end;

        result := true; //if everything before succeeded
 end;

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
function TProcessThread.Run1Command(cmdLine: string): boolean;
{start the desired command line and manage various normal and error message
returns false if it fails}
begin
    result := RunCmdLine(cmdLine); //launch the command line
    if result = false then begin //if failed
        TellErrorOccurs(cmdLine + rstrTASK_ABORTED, taskNber); //fire the error event
        //TellCmdDone(rstrTASK_FAILS); //fire one progression message
    end;
end;

//--------------------------------------------------------------------------------------------------------
function TProcessThread.isEnoughSpaceFor(sourcePath: string; destPath: string): boolean;
{get the file size of the source path to have an approximation of needed space for destination
then extract the available free space on the drive that would received the destination file,
if the space is not sufficiant, manage an error an return false}
 var
    sourceF: file;
    fSize: int64;

 begin
    result := false; //init

    //get the source file size:
    AssignFile(sourceF, sourcePath);
    reset(sourceF, 1024);
    //recsize is set to 1024 to have a filesize below in Ko (in o, the size can't be expressed as an integer !)
    fSize := fileSize(sourceF);
    CloseFile(sourceF);

    //get the destination drive nber:
    ChDir(leftstr(destPath,1)+':\'); {extract the destination drive from the path
        to be the current dir (to prepare use of diskFree function below}

    //check that free space is sufficiant:
    if int(fSize * 1024 * 1.15) < DiskFree(0) then begin {no problem free space is sufficiant
            (a +15% margin is used to make this procedure compatible with m2v file
             that will generate a VOB file larger after remux)}
        result := true;
    end else begin
        result := false;
        TellErrorOccurs(rstrNOT_ENOUGH_SPACE + destPath, taskNber);
    end;
 end;



//--------------------------------------------------------------------------------------------------------
procedure TProcessThread.TellCmdStarted(ProgressMessage: string;
    taskNber: integer);
{start the procedure assigned to the OnCmdStarted event}
begin
  if Assigned(FOnCmdStarted) then { the procdure is assigned ?... }
    FOnCmdStarted(Self, ProgressMessage, taskNber); { ... execute it }
end;

//--------------------------------------------------------------------------------------------------------
procedure TProcessThread.TellCmdDone(ProgressMessage: string; taskNber: integer);
{start the procedure assigned to the OnCmdDone event}
begin
  if Assigned(FOnCmdDone) then { the procdure is assigned ?... }
    FOnCmdDone(Self, ProgressMessage, taskNber); { ... execute it }
end;

//--------------------------------------------------------------------------------------------------------
procedure TProcessThread.TellTaskStarted(ProgressMessage: string; taskNber: integer);
{start the procedure assigned to the OnTaskStarted event}
begin
  if Assigned(FOnTaskStarted) then { the procdure is assigned ?... }
    FOnTaskStarted(Self, ProgressMessage, taskNber); { ... execute it }
end;



//--------------------------------------------------------------------------------------------------------
procedure TProcessThread.TellErrorOccurs(errorMessage: string; taskNber: integer);
{start the procedure assigned to the OnError event}
begin
  if Assigned(FOnError) then { the procdure is assigned ?... }
    FOnError(Self, errorMessage, taskNber); { ... execute it }
end;

//--------------------------------------------------------------------------------------------------------
destructor TProcessThread.Destroy;
{automatically executed when the thread ends}
begin
  FreeAndNil(FEvent); { destroy the event }
  inherited;
end;


//--------------------------------------------------------------------------------------------------------
function TProcessThread.isFileSizeCorrect(theFileName: string; expectedSizeKo: real; tolerance: real): boolean;
{check that the size of file named "theFileName" has a size similar to what
it is expected "expectedSizeKo".
Otherwise it means that en error occured.
This function handle also the error management if needed.
A tolerance "tolerance" expressed in %, is used for the comparison}
 var
    fSize: real; //its size in Ko
 begin
    result := true; //init to ok
    fSize := getSizeOfFile(theFileName); //get the source file size

    //compare it :
    if fSize < ((1- tolerance/100) * expectedSizeKo) then begin //if the size doesn't match with expectation (within the tolerance)
        result := false;
        TellErrorOccurs(rstrFILE_SIZE_DOESNT_MATCH +
            FormatFloat ('#,### "Ko"', fSize) + '/' +
            FormatFloat ('#,### "Ko"', expectedSizeKo) + ' (' +
            FormatFloat ('##.## "%"', tolerance) + ') )', taskNber);
    end;

 end;

//--------------------------------------------------------------------------------------------------------
function TProcessThread.isDirectorySizeCorrect(theDirName: string; expectedSizeKo: real; tolerance: real): boolean;
{This function is similar to isFileSizeCorrect, except it works with a full directory
and not only a single file.
However there is some restrictions :
It will only look at files named vts_01_n.vob inside the directory to calculate the size.

check that the size of the directory named "theDirName" has a size similar to what
it is expected "expectedSizeKo".
Otherwise it means that en error occured.
This function handle also the error management if needed.
A tolerance "tolerance" expressed in %, is used for the comparison
}
 var
    fSize: real; //the total size of the directory in Ko (in fact only VOB file sizes summation)
 begin
    result := true; //init to ok

    //sum all vob files size :
    fSize := getSizeOfDir(theDirName + '\');

    //compare it :
    if fSize < ((1- tolerance/100) * expectedSizeKo) then begin //if the size doesn't match with expectation (within the tolerance)
        result := false;
        TellErrorOccurs(rstrDIR_SIZE_DOESNT_MATCH +
            FormatFloat ('#,### "Ko"', fSize) + '/' +
            FormatFloat ('#,### "Ko"', expectedSizeKo) + ' (' +
            FormatFloat ('##.## "%"', tolerance) + ') )', taskNber);
    end;

 end;

 //--------------------------------------------------------------------------------------------------------
function TProcessThread.RunCmdLine(const CmdLine: string; WindowState: Word = SW_NORMAL): Boolean;
{to start an external application from a command line

argument :
    CmdLine = string that include the command line (application path + parameters)
    WindowState = every constant named who's name begin by SW_
}

{only to keep information's on that function (for future uses ...) :
function ShellExecute(hwnd: Integer;
    lpOperation: String;
    lpFile: String;
    lpParameters: String;
    lpDirectory: String;
    nShowCmd: integer): integer; external 'shell32.dll'; //To start an external application }
var
    SUInfo: TStartupInfo;
    ProcInfo: TProcessInformation;
    CreateOK : Boolean; //true if the command line was executed
    rep: integer; //user answer
begin
  { Enclose filename in quotes to take care of
    long filenames with spaces. }
    FillChar(SUInfo, SizeOf(SUInfo), #0);
    with SUInfo do
    begin
        cb := SizeOf(SUInfo);
        dwFlags := STARTF_USESHOWWINDOW;
        wShowWindow := WindowState;
    end;
    CreateOK := CreateProcess(nil, PChar(CmdLine), nil, nil, False,
        CREATE_NEW_CONSOLE or
        NORMAL_PRIORITY_CLASS, nil,
        nil {PChar(ExtractFilePath(Filename))},
        SUInfo, ProcInfo);

    if CreateOK then begin
        WaitForSingleObject(ProcInfo.hProcess, INFINITE);//wait the child process is finished before continuing
    end else begin //if the cde line fails
        rep := MessageDlg(rstrTHIS_CDE_LINE_FAILS + CmdLine, mtWarning , [mbOk], 0);
    end;

    CloseHandle(ProcInfo.hProcess);
    CloseHandle(ProcInfo.hThread);
    result := CreateOK;

end;

end.











