unit progressThreadUnit;
{
    This object TProgressThread fire regularely an event (OnProgress) to tell the main unit
    how big is the destination file (or dir).
    Therefore the main unit is able to update a progress bar to monitor the current
    task progression.
    It is run under a specific thread to avoid the user to loos control on the program
    while tasks are running.
______________________________________________________________________________
}
{TurboVOB - A freeware to prepare DTH7000's video for DVD burning on computer under windows.
Copyright (C) 2005 Didier CADIOU

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see gpl.txt file); if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.}

interface

uses
  //personal :
  globalMethods,
  //generic :
  Classes, SyncObjs, SysUtils, Dialogs, language, Windows, DateUtils,
  strUtils, forms;

type TProgressEvent = procedure (Sender: TObject; progress: integer) of object;
type

  TprogressThread = class(TThread)
  private
    path: string;
    expected: real;
    isDirectory: boolean;
    FEvent : TEvent;  { internal event }
    FOnProgress: TProgressEvent;  {event Procedure that will be called each time a cmd line is started}

  protected
    procedure TellUpdateProgress(progress: integer); { Wrapper for FOnProgress}
    procedure Execute; override;
  public
    constructor Create(CreateSuspended: Boolean; pathName: string; expectedSize: real; isDir: boolean);
    //events fired regularely during process :
    property OnProgress: TProgressEvent read FOnProgress write FOnProgress; {regularely while the task is running}
    destructor Destroy();
  end;


implementation

//--------------------------------------------------------------------------------------------------------
constructor TProgressThread.Create(CreateSuspended: Boolean;
    pathName: string; //path of destination file or dir, for which the size will be monitor
    expectedSize: real; //the expected size of the destination at the end of the current operation
    isDir: boolean); //true if the destination is not only a file but a complete dir (ex: VIDEO_TS)

begin
    inherited Create(CreateSuspended);
    Priority := tpLower;

    FEvent := TEvent.Create(nil, False, False, '');

    //affect local variables from parameters :
    path := pathName;
    expected := expectedSize;
    isDirectory := isDir;
end;

//--------------------------------------------------------------------------------------------------------
procedure TProgressThread.Execute;
var
    lastUpdateTime: TDateTime; //time when the last onProgress event was fired
    delay: TDateTime; //delay (in days) between 2 updates of the progress bar
    progression: real; //% of completion (current size compared to expected size, should 100% at the end)
begin
    delay := 1/(24*3600); //2 sec.

    lastUpdateTime := now;
    while not Terminated do begin //loop until the current command line execution is terminated
        if (now - lastUpdateTime) >= delay then begin //if the last update is too "old" launch a new one
            if isDirectory = false then begin //destination is file
                progression := 100 * (getSizeOfFile(path) / expected);
            end else begin //destination is a directory (usually VIDEO_TS)
                progression := 100 * (getSizeOfDir(path) / expected);
            end;

            if progression > 100 then progression := 100; //clamped to 100%
            TellUpdateProgress(round(progression));
            lastUpdateTime := now;
        end;
    end;


end;

//--------------------------------------------------------------------------------------------------------
procedure TProgressThread.TellUpdateProgress(Progress: integer);
{start the procedure assigned to the OnProgress event

Progress is the percentage of completion (in %)}
begin
  if Assigned(FOnProgress) then { the procdure is assigned ?... }
    FOnProgress(Self, progress); { ... execute it }
end;


//--------------------------------------------------------------------------------------------------------
destructor TProgressThread.Destroy;
{automatically executed when the thread ends}
begin
  FreeAndNil(FEvent); { destroy the event }
  inherited;
end;



end.











