unit taskDialog;

{
______________________________________________________________________________
    manage the dialog that will be used to define accuratly
    every task characteristics 
______________________________________________________________________________
}
{TurboVOB - A freeware to prepare DTH7000's video for DVD burning on computer under windows.
Copyright (C) 2005 Didier CADIOU

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see gpl.txt file); if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, task, language, BrowseDialog, ExtCtrls, StrUtils,
  ComCtrls;

type
  TFormTaskManager = class(TForm)
    btnOK: TButton;
    btnCancel: TButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    Panel_source: TPanel;
    Label4: TLabel;
    txt_origVOBfilePath: TEdit;
    chb_deleteOrigVob: TCheckBox;
    btn_BrowseForOriginalVOB: TButton;
    Panel_streams: TPanel;
    Label1: TLabel;
    txt_m2vFilePath: TEdit;
    chb_deleteStreams: TCheckBox;
    btn_BrowseForVideoStream: TButton;
    btn_BrowseForAudioStream: TButton;
    Panel_remuxed: TPanel;
    Label3: TLabel;
    txt_VOBFilePath: TEdit;
    chb_deleteVob: TCheckBox;
    btn_BrowseForVob: TButton;
    chb_author: TCheckBox;
    chb_Demux: TCheckBox;
    Panel_chapter: TPanel;
    Label2: TLabel;
    btn_ImportChapter: TButton;
    edt_chapterList: TRichEdit;
    txt_ac3FilePath: TEdit;
    Panel_folder: TPanel;
    Label5: TLabel;
    txt_DVDfolderPath: TEdit;
    btn_BrowseForDVD: TButton;
    chb_remux: TCheckBox;
    ImDemuxArrow: TImage;
    ImRemuxArrow: TImage;
    ImAuthorArrow: TImage;
    ImChapterArrow: TImage; //dialog to choose a destination folder
    procedure btnCancelClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure disableUselessControls(Sender: TObject);
    procedure btn_BrowseForVideoStreamClick(Sender: TObject);
    procedure btn_BrowseForAudioStreamClick(Sender: TObject);
    procedure btn_BrowseForDVDClick(Sender: TObject);
    procedure btn_BrowseForVobClick(Sender: TObject);
    procedure btn_BrowseForOriginalVOBClick(Sender: TObject);
    procedure btn_ImportChapterClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure chb_deleteOrigVobClick(Sender: TObject);
    procedure chb_deleteStreamsClick(Sender: TObject);
    procedure chb_deleteVobClick(Sender: TObject);
  private
    { Dclarations prives }
    theTask: Ttask;
    rep: integer; //for user answer
    folderDialog: TBrowseDialog;
    procedure importChapter(chapterFile: TFileName); //to import chapter location from a txt file
    function getChapterList(): string; //to read the chapter list from the form
    procedure openVideoStream(filePath: string); //done just after a video stream file has been selected in the task manager form


  public
    { Dclarations publiques }
    constructor Create(AOwner: TComponent; var aTask: Ttask);
    procedure BrowseForOriginalVOB;
  end;

var
  FormTaskManager: TFormTaskManager;

implementation
{$R *.dfm}
uses
    main;

//--------------------------------------------------------------------------------------------------------
constructor TFormTaskManager.Create(AOwner: TComponent; var aTask: Ttask);
{argument :
AOwner => parent form ?
newTask => task object (passed by variable) that will be updated by this dialog}
begin
    Inherited create(AOwner);
    theTask := aTask;

    chb_Demux.Checked := aTask.ToBeDemuxed;
    chb_remux.Checked := aTask.ToBeRemuxed;
    chb_author.Checked := aTask.ToBeAuthored;
    txt_origVOBfilePath.Text := aTask.originalVobPath;
    txt_m2vFilePath.Text := aTask.videoStreamPath;
    txt_ac3FilePath.Text := aTask.audioStreamPath;
    txt_VOBFilePath.Text := aTask.VobPath;
    txt_DVDfolderPath.Text := aTask.destinationFolder;
    chb_deleteOrigVob.Checked := aTask.deleteOriginalVob;
    chb_deleteStreams.Checked := aTask.deleteStreams;
    chb_deleteVob.Checked := aTask.deleteRemuxedVob;

    disableUselessControls(self);


end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.FormShow(Sender: TObject);
{automatically read chapters location if the expected file exists}
begin
    if length(theTask.chapterListStr)>0 then begin //if chapter has been already defined
        edt_chapterList.Text := AnsiReplaceText(theTask.chapterListStr, ',', #13#10); //load it
    end else begin //if chapter aren't currently defined
        {if (FileExists(ExtractFilePath(txt_origVOBfilePath.Text) + 'CellTimes.txt') = true) then begin //if a celltimes.txt file exits
            importChapter(ExtractFilePath(txt_origVOBfilePath.Text) + 'CellTimes.txt'); //load it
        end;}
    end;
end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.btnCancelClick(Sender: TObject);
begin
    ModalResult := mrCancel;
end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.btnOKClick(Sender: TObject);
begin
    //various checks :
    //potential overwrite of source file
    if txt_origVOBfilePath.text = txt_VOBFilePath.text then begin
        rep := MessageDlg(rstrVOB_FILE_HAS_SAME_NAME, mtWarning, [mbAbort, mbIgnore], 0);
        if rep = mrAbort then exit;
    end;

    //check that source file don't miss
    if chb_Demux.Checked = true then begin //demux means source is the VOB file
        if FileExists(txt_origVOBfilePath.Text) = false then begin //check it exists
            MessageDlg(txt_origVOBfilePath.Text + rstrFILE_MISSING, mtWarning, [mbOK], 0);
            exit;
        end;
    end;

    if (chb_Demux.Checked = false) and
        (chb_remux.Checked = true) then begin //remux without demux, means source are streams files
        if FileExists(txt_m2vFilePath.Text) = false then begin //check video streams exixts
            MessageDlg(txt_m2vFilePath.Text + rstrFILE_MISSING, mtWarning, [mbOK], 0);
            exit;
        end;
        if FileExists(txt_ac3FilePath.Text) = false then begin //same check for audio
            MessageDlg(txt_ac3FilePath.Text + rstrFILE_MISSING, mtWarning, [mbOK], 0);
            exit;
        end;
    end;

    if (chb_Demux.Checked = false) and
        (chb_remux.Checked = false) and
        (chb_author.Checked = true)  then begin //author only, means source is remuxed VOB file
        if FileExists(txt_VOBFilePath.Text) = false then begin //check it exixts
            MessageDlg(txt_VOBFilePath.Text + rstrFILE_MISSING, mtWarning, [mbOK], 0);
            exit;
        end;
    end;

    //validate choices :
    with theTask do begin
        originalVobPath := txt_origVOBfilePath.Text;
        if Panel_source.Visible = false then
            originalVobPath := '';

        videoStreamPath := txt_m2vFilePath.Text;
        audioStreamPath := txt_ac3FilePath.Text;
        if Panel_streams.Visible = false then begin
            videoStreamPath := '';
            audioStreamPath := '';
        end;

        VobPath := txt_VOBFilePath.Text;
        if Panel_remuxed.Visible = false then
            VobPath := '';

        if Panel_folder.Visible = true then begin
            destinationFolder := txt_DVDfolderPath.Text;
            chapterListStr := getChapterList;
        end else begin //if no authoring
            destinationFolder := '';
            chapterListStr := '';
        end;

        ToBeDemuxed := chb_Demux.Checked;
        ToBeRemuxed := chb_remux.Checked;
        ToBeAuthored := chb_author.Checked;
        deleteOriginalVob := chb_deleteOrigVob.Checked;
        deleteStreams := chb_deleteStreams.Checked;
        deleteRemuxedVob := chb_deleteVob.Checked;

        {force directory (check that directory exists, if not create it)
        of destination to avoid missing destination folder :}
        if chb_author.Checked = true then begin //destination is re-authored VIDEO_TS folder
            if ForceDirectories(txt_DVDfolderPath.Text) = false then begin
                MessageDlg('"' + txt_DVDfolderPath.Text + '" ' + rstrDIRECTORY_CANT_BE_CREATED, mtWarning, [mbOK], 0);
                exit;
            end;
        end else begin //no authoring
            if chb_remux.Checked = true then begin //destination is remuxed VOB file
                if ForceDirectories(ExtractFileDir(txt_VOBFilePath.Text)) = false then begin
                    MessageDlg('"' + txt_VOBFilePath.Text + '" ' + rstrDIRECTORY_CANT_BE_CREATED, mtWarning, [mbOK], 0);
                    exit;
                end;
            end else begin //no remuxing
                if chb_remux.Checked = true then begin //destination is demuxed stream files
                    if ForceDirectories(ExtractFileDir(txt_m2vFilePath.Text)) = false then begin
                        MessageDlg('"' + txt_m2vFilePath.Text + '" ' + rstrDIRECTORY_CANT_BE_CREATED, mtWarning, [mbOK], 0);
                        exit;
                    end;
                end;
            end;
        end;

    end; //with theTask



    ModalResult := mrOK;
end;
//--------------------------------------------------------------------------------------------------------
function TFormTaskManager.getChapterList(): string;
{convert the chapter list in the form to a string with all values separated by ','}
 var
    ix: integer; //loop index
    OneSec: real; //1 seconde expressed in day
    chapterMark: string; //the chapter mark currently analyzed
 begin
        result := ''; //init
        OneSec := 1/(24*3600);
        for ix := 0 to (edt_chapterList.Lines.Count -1) do begin //sweep through all chapter marks
            chapterMark := edt_chapterList.Lines[ix];

            //convert mark locations if needed (and possible) :
            if LastDelimiter(':',chapterMark) = 0 then begin //no ':' separator => it could means that marks are expressed as frame number
                {try to convert the string as if it is a frame number
                This conversion it is done in relation with video standard}
                try
                    if optStandard = 'PAL' then begin
                        chapterMark := formatDateTime('hh":"nn":"ss',((StrToFloat(chapterMark))/25)*OneSec); //in PAL = 25 frames per second
                    end else begin //NTSC
                        chapterMark := formatDateTime('hh":"nn":"ss',((StrToFloat(chapterMark))/29.97)*OneSec); //in NTSC = 29.97 frames per second
                    end;
                except
                    {if the time conversion fails, an exception will be generated but not catched,
                    therefore the program continue and simply ignore this chapter location}
                end;
            end;//if no ':' separator

            if LastDelimiter(':',chapterMark) = 9 then begin //no ':' separator is expected at position 8, except if time is expressed as hh:mm:ss:msec
                // instead of hh:mm:ss.msec (mpeg2schnitt do that sometime)
                {try to convert the string to replace the last ':' by a '.'}
                chapterMark := leftStr(chapterMark, 8) + '.' + rightStr(chapterMark, length(chapterMark)-9);
            end;



            //Add the current mark to the output string :
            try
                StrToTime(chapterMark); //only to generate a EConvertError if the string is not a valid time
                result := result + chapterMark + ',';
            except //if an error happened while converting the chapter location
                {if the time conversion fails in line above, an exception will be generated but not catched,
                therefore the program continue and simply ignore this corrupted chapter location}
            end;

        end; //sweep through all chapter marks

        result := LeftStr(result, LastDelimiter(',', result)-1);
                //remove all characters at the end of the string,
                //starting after the last ',' delimiter (included)
 end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.btn_BrowseForOriginalVOBClick(Sender: TObject);
begin
    BrowseForOriginalVOB;
end;
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure TFormTaskManager.BrowseForOriginalVOB;
var
    FileExtension: string; //file name extension (related to the file type)
begin
    with OpenDialog1 do begin
        if length(txt_origVOBfilePath.Text)>0 then begin
            FileName := txt_origVOBfilePath.Text; //apply previous filename if it exists
        end;
        DefaultExt := 'VOB';
        Filter := 'Fichiers Mpeg2 DVD (*.VOB)|*.VOB|Fichiers Mpeg2 (*.mpg)|*.mpg|Fichiers Mpeg2 (*.mpeg)|*.mpeg|video stream (*.mpv)|*.mpv|video stream (*.m2v)|*.m2v';
        Title := rstrSELECT_VOB_FILE_SOURCE;
        if Execute then begin
            FileExtension := lowercase(ExtractFileExt(FileName));
            if (FileExtension = '.vob') or
             (FileExtension = '.mpg') or
             (FileExtension = '.mpeg') then begin
                txt_origVOBfilePath.Text := FileName;
                //autofill the other fields :
                if (FileExtension = '.vob') then begin
                    txt_m2vFilePath.Text := ChangeFileExt(filename, '.m2v');
                    txt_ac3FilePath.Text := ChangeFileExt(filename, '.ac3'); //assuming audio is ac3
                end else begin //not a pure DVD file (generic mpeg)
                    txt_m2vFilePath.Text := ChangeFileExt(filename, '.mpv');
                    txt_ac3FilePath.Text := ChangeFileExt(filename, '.mp2'); //assuming audio is mp2
                end;
                txt_VOBFilePath.Text := ChangeFileExt(ExtractFilePath(filename) + 'remuxed' + ExtractFileName(filename), '.VOB');
                txt_DVDfolderPath.Text :=
                    leftStr(ExtractFilePath(filename), length(ExtractFilePath(filename))-1); //path without the '/'
                if (FileExists(ExtractFilePath(txt_origVOBfilePath.Text) + 'CellTimes.txt') = true) then begin //if a celltimes.txt file exits
                    importChapter(ExtractFilePath(txt_origVOBfilePath.Text) + 'CellTimes.txt'); //load it
                end;
            end else begin //an already demuxed video stream is selected (the demux is therefore not needed)
                //update display because the source file is no longer a vob file :
                chb_Demux.Checked := false;
                disableUselessControls(self);

                //fill form fields from user answer :
                openVideoStream(FileName);
            end;

        end;
    end;
end;

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure  TFormTaskManager.btn_BrowseForVideoStreamClick(Sender: TObject);
begin
    if chb_Demux.Checked = true then begin //demux will be done, demuxed streams doesn't already exists
        with SaveDialog1 do begin
            FileName := 'VTS_01_1.m2v'; //default filename
            if length(txt_m2vFilePath.Text)>0 then begin
                FileName := txt_m2vFilePath.Text; //apply previous filename if it exists
            end;
            DefaultExt := 'm2v';
            Filter := 'video stream file (*.m2v)|*.m2v|video stream file (*.mpv)|*.mpv';
            Title := rstrSELECT_STREAM_FILE_DESTINATION;
            if Execute then begin
                txt_m2vFilePath.Text := FileName; //select destination
                //autofill the other fields :
                if LowerCase(ExtractFileExt(FileName)) = '.m2v' then begin
                    txt_ac3FilePath.Text := ChangeFileExt(FileName, '.ac3'); //automatically update also audio destination
                end else begin //.mpv probably
                    txt_ac3FilePath.Text := ChangeFileExt(FileName, '.mp2'); //automatically update also audio destination
                end;
                txt_VOBFilePath.Text := ChangeFileExt(ExtractFilePath(filename) + 'remuxed' + ExtractFileName(filename), '.VOB');
                txt_DVDfolderPath.Text :=
                    leftStr(ExtractFilePath(filename), length(ExtractFilePath(filename))-1); //path without the '/'
            end;
        end;
    end else begin //no demux will be done, demuxed streams exists and will be used for remuxing
        with OpenDialog1 do begin
            if length(txt_m2vFilePath.Text)>0 then begin
                FileName := txt_m2vFilePath.Text; //apply previous filename if it exists
            end;
            DefaultExt := 'm2v';
            Filter := 'video stream file (*.m2v)|*.m2v|video stream file (*.mpv)|*.mpv';
            Title := rstrSELECT_STREAM_FILE_SOURCE;
            if Execute then begin
                openVideoStream(FileName);
            end; //if execute
        end; //with opendialog1
    end; //chb_Demux.Checked
end;

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure TFormTaskManager.openVideoStream(filePath: string);
begin
    txt_m2vFilePath.Text := filePath; //select source

    //autofill the other fields :
    if FileExists(ChangeFileExt(filePath, '.ac3')) = true then begin//also select audio source if it exists and is named as the video stream
        txt_ac3FilePath.Text := ChangeFileExt(filePath, '.ac3');
    end else if FileExists(ChangeFileExt(filePath, '.mp2')) = true then begin//also select audio source if it exists and is named as the video stream
        txt_ac3FilePath.Text := ChangeFileExt(filePath, '.mp2');
    end else begin //if it doesn't exists as expected
        txt_ac3FilePath.Text := leftStr(ExtractFilePath(filePath), length(ExtractFilePath(filePath))-1); //pre-enter only the folder path without the '/' car. (the user will browse manually to find it
    end; //if audio file exits

    txt_VOBFilePath.Text := ChangeFileExt(ExtractFilePath(filePath) + 'remuxed' + ExtractFileName(filepath), '.VOB');
    txt_DVDfolderPath.Text :=
        leftStr(ExtractFilePath(filePath), length(ExtractFilePath(filePath))-1); //path without the '/'

    if FileExists(ExtractFilePath(filePath) + 'CellTimes.txt') = true then begin
        importChapter(ExtractFilePath(filePath) + 'CellTimes.txt');
    end;
end;

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure TFormTaskManager.btn_BrowseForAudioStreamClick(Sender: TObject);
begin
    if chb_Demux.Checked = true then begin //demux will be done, demuxed streams doesn't already exists
        with SaveDialog1 do begin
            FileName := 'VTS_01_1.ac3'; //default filename
            if length(txt_ac3FilePath.Text)>0 then begin
                FileName := txt_ac3FilePath.Text; //apply previous filename if it exists
            end;
            DefaultExt := 'ac3';
            Filter := 'audio stream file (*.ac3)|*.ac3|audio stream file (*.mp2)|*.mp2';
            Title := rstrSELECT_STREAM_FILE_DESTINATION;
            if Execute then
                txt_ac3FilePath.Text := FileName; //select destination
        end;
    end else begin //no demux will be done, demuxed streams exists and will be used for authoring
        with OpenDialog1 do begin
            if length(txt_ac3FilePath.Text)>0 then begin
                FileName := txt_ac3FilePath.Text; //apply previous filename if it exists
            end;
            DefaultExt := 'ac3';
            Filter := 'audio stream file (*.ac3)|*.ac3|audio stream file (*.mp2)|*.mp2';
            Title := rstrSELECT_STREAM_FILE_SOURCE;
            if Execute then
                txt_ac3FilePath.Text := FileName; //select source
        end;
    end;
end;
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure TFormTaskManager.btn_BrowseForVobClick(Sender: TObject);
begin
    if chb_remux.Checked = true then begin //remux is done, so remuxed VOB file is a temporary file (to be saved rather than to be openned)
        with SaveDialog1 do begin
            FileName := 'VTS_01_1.VOB'; //default filename
            if length(txt_VOBFilePath.Text)>0 then begin
                FileName := txt_VOBFilePath.Text; //apply previous filename if it exists
            end;
            DefaultExt := 'VOB';
            Filter := 'video file (*.VOB)|*.VOB';
            Title := rstrSELECT_VOB_FILE_DESTINATION;
            if Execute then begin
                if filename = txt_origVOBfilePath.Text then begin
                    rep := MessageDlg(rstrCAUTION_ORIGINAL_AND_DESTINATION_ARE_EQUAL, mtWarning, [mbYes, mbNo], 0);
                    if rep = mrNo then exit;
                end;
                txt_VOBFilePath.Text := FileName;
                txt_DVDfolderPath.Text :=
                    leftStr(ExtractFilePath(filename), length(ExtractFilePath(filename))-1); //path without the '/'
            end;
        end;
    end else begin //no remux is done, so VOB file is the source to be opened
        with OpenDialog1 do begin
            if length(txt_VOBFilePath.Text)>0 then begin
                FileName := txt_VOBFilePath.Text; //apply previous filename if it exists
            end;
            DefaultExt := 'VOB';
            Filter := 'video file (*.VOB)|*.VOB';
            Title := rstrSELECT_VOB_FILE_SOURCE;
            if Execute then begin
                txt_VOBFilePath.Text := FileName;
                txt_DVDfolderPath.Text :=
                    leftStr(ExtractFilePath(FileName), length(ExtractFilePath(FileName))-1); //path without the '/'
                if (FileExists(ExtractFilePath(txt_VOBFilePath.Text) + 'CellTimes.txt') = true) then begin //if a celltimes.txt file exits
                    importChapter(ExtractFilePath(txt_VOBFilePath.Text) + 'CellTimes.txt'); //load it
                end;
            end; //if execute
        end; //with opendialog1
    end; //chb_remux.Checked
end;
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure TFormTaskManager.btn_BrowseForDVDClick(Sender: TObject);
begin
    folderDialog := TBrowseDialog.Create(FormTaskManager);
    folderDialog.Title := rstrSELECT_DVD_FOLDER_DESTINATION;
    if length(txt_DVDfolderPath.Text)>0 then begin
        folderDialog.Folder := txt_DVDfolderPath.Text; //apply previous folder name if it exists
    end;

    if folderDialog.Execute then //open a specific dialog to ask where is the destination forder
        txt_DVDfolderPath.Text := folderDialog.Folder;

    folderDialog.Free;
end;


//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.disableUselessControls(Sender: TObject);
begin
    //depending on weither demuxing is done or not :
    ImDemuxArrow.Visible := chb_Demux.Checked;
    Panel_source.Visible := chb_Demux.Checked;

    //hide stream panel if nothing is asked for it :
    Panel_streams.Visible := false;
    if (chb_Demux.Checked = true) or (chb_remux.Checked = true) then begin
        Panel_streams.Visible := true;
    end;

    //depending on weither remuxing is done or not :
    ImRemuxArrow.Visible := chb_remux.Checked;
    chb_deleteStreams.Enabled := chb_remux.Checked; //streams can be deleted only if they are not the destination (if remuxing is done afterward)
    if (chb_Demux.Checked = true) and
        (chb_remux.Checked = false) then
            chb_author.Checked := false; //authoring can't be done without remuxed vob file after demux

    //hide remuxed panel if nothing is asked for it:
    Panel_remuxed.Visible := false;
    if (chb_remux.Checked = true) or (chb_author.Checked = true) then begin
        Panel_remuxed.Visible := true;
    end;

    //depending on weither authoring is done or not :
    ImAuthorArrow.Visible := chb_author.Checked;
    ImChapterArrow.Visible := chb_author.Checked;
    Panel_folder.Visible := chb_author.Checked;
    Panel_chapter.Visible := chb_author.Checked;
    chb_deleteVob.Enabled := chb_author.Checked; //remuxed can be deleted only if it is not the destination (if authoring is done afterward)


end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.btn_ImportChapterClick(Sender: TObject);
{Browse to a txt file that contains chapters location
    and import them}
begin
    //open the dialog to choose the file :
    with OpenDialog1 do begin
        FileName := txt_m2vFilePath.Text;
        DefaultExt := 'txt';
        Filter := 'text file (*.txt)|*.txt';
        Title := rstrSELECT_CHAPTER_FILE_SOURCE;
        if Execute then begin
            importChapter(FileName);
        end;
    end;
end;
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure TFormTaskManager.importChapter(chapterFile: TFileName);
{import chapter locations from a file chapterFile to the edit control}
var
    txtFile: textfile; //source file handle
    txt: string; //1 lines from the txt file
    txts: string; //all lines from the txt file, separated by carr. ret. + line feed car
begin
    AssignFile(txtFile, chapterFile); //associate file name to file variable
    reset(txtFile); //open the file

    //read the file to the edit control :
    txts := ''; //init
    while not Eof(txtFile) do begin
        Readln(txtFile, txt);
        txts := txts + txt + #13#10
    end;
    edt_chapterList.Text := txts;
    theTask.chapterListStr := getChapterList ; //affect chapters string to the appropriate task property
        //and benefit of chapter conversion if needed, which is included inside getChapterList fct
    edt_chapterList.Text := AnsiReplaceText(theTask.chapterListStr, ',', #13#10); //replace the chapter display
        //by potentialy converted chapters marks

    CloseFile(txtFile);
end;


//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.chb_deleteOrigVobClick(Sender: TObject);
begin
    if chb_deleteOrigVob.Checked = true then begin
        MessageDlg(rstrCAUTION_PT_OF_NO_RETURN, mtWarning, [mbOK], 0);
    end;
end;
//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.chb_deleteStreamsClick(Sender: TObject);
begin
    if (chb_deleteStreams.Checked = true) and
        (chb_Demux.checked = false) then begin //if no demux asked, it means that streams are source files
            //and must be preserved otherwise a risk of definitive loss exists
                MessageDlg(rstrCAUTION_PT_OF_NO_RETURN, mtWarning, [mbOK], 0);
    end;
end;
//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.chb_deleteVobClick(Sender: TObject);
begin
    if (chb_deleteVob.Checked = true) and //if this file should be deleted
        (chb_Demux.checked = false) and
        (chb_remux.checked = false) then begin //and it is the source file,
            //it must be preserved otherwise a risk of definitive loss exists
                MessageDlg(rstrCAUTION_PT_OF_NO_RETURN, mtWarning, [mbOK], 0);
    end;
end;

end.
